package Logoved::DB::Rule::Matcher::Multiline;
use Moo;
extends 'Logoved::DB::Rule::Matcher';

has 'multiline_pattern_begin' => (is => 'ro');
has 'multiline_pattern_body' => (is => 'ro');
has 'multiline_pattern_end' => (is => 'ro');

has 'multiline_begin_groups' => (is => 'rw');
has 'multiline_body_groups' => (is => 'rw');
#has 'multiline_end_groups' => (is => 'rw');

has 'lineno_begin' => (is => 'rw');
has 'lineno_end' => (is => 'rw');
has 'line' => (is => 'rw');
has '__is_internal_state_matching_multiline_pattern' => (is => 'rw');

our $verbose=0;

sub match {
    my ($self, $line, $chomped_line)=@_;
    my $source=$self->source;
    if (! $self->{__is_internal_state_matching_multiline_pattern}) {
	my $multipattern_begin=$self->multiline_pattern_begin;
	my @begin_groups= $chomped_line =~ $multipattern_begin;
	if (@begin_groups>0) {
	    $self->{__is_internal_state_matching_multiline_pattern}=1;
	    $self->lineno_begin($source->lineno);
	    $self->line($line);
	    $self->multiline_begin_groups(\@begin_groups) if $#+ >0;
	    $self->multiline_body_groups([]);
	    #$self->multiline_end_groups([]);
	    warn 'Logoved: MLBEGIN: ', $source->name, ' ',$source->file, ': ', $line if $verbose;
	}
    } else {
	my @end_groups=$chomped_line =~ $self->multiline_pattern_end;
	if (@end_groups>0) {
	    $self->line($self->line() . $line);
	    $self->lineno_end($source->lineno);
	    warn 'Logoved: MLEND: ', $source->name, ' ',$source->file, ': ', $line if $verbose;
	    $self->{__is_internal_state_matching_multiline_pattern}=0;
	    my $lineno_begin=$self->lineno_begin;
	    my $lineno_end=$self->lineno_end;
	    my @args=(
		rule => $self->rule,
		logfile => $source->file,
		line => $self->line,
		lineno => $lineno_begin.'-'.$lineno_end,
		multiline_begin_groups => $self->multiline_begin_groups,
		multiline_body_groups => $self->multiline_body_groups,
		);
	    # regex has $#+ groups, see @+, @LAST_MATCH_END
	    # http://perldoc.perl.org/perlvar.html#@LAST_MATCH_END
	    push @args, multiline_end_groups => \@end_groups if $#+ >0;
	    return Logoved::DB::Rule::Match->new(@args);
	    
	} else {
	    my $multipattern_body=$self->multiline_pattern_body;
	    if ($multipattern_body) {
		my $is_body_matched=0;
		foreach my $pattern (@$multipattern_body) {
		    my @body_groups = $chomped_line =~ $pattern;
		    if (@body_groups>0) {
			$self->line($self->line() . $line);
			push @{$self->multiline_body_groups}, @body_groups if $#+ >0;
			warn 'Logoved: ML BODY: ', $source->name, ' ',$source->file, ': ', $line if $verbose;
			$is_body_matched=1;
			last;
		    }
		}
		if (not $is_body_matched) {
		    warn 'Logoved: ML BODY: not a match: ', $source->name, ' ',$source->file, ': ', $line if $verbose;
		    $self->{__is_internal_state_matching_multiline_pattern}=0;
		}
	    }
	}
    }
    return;
}

1;
